/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*- */

#ifndef GENERAL__SHARED_PTR_HH
#define GENERAL__SHARED_PTR_HH
   
#if defined(SWIG)
%include <general/SharedPtr.i>
#else
#include <tr1/memory>

#define SHARED_PTR_NAMESPACE std::tr1

namespace General
{
#if !defined(SWIG)
#if 1
#define SharedPtr shared_ptr
#define SharedConstPtr shared_ptr
#define WeakPtr weak_ptr

#define StaticPointerCast static_pointer_cast
#define DynamicPointerCast dynamic_pointer_cast

#define LDAS_SHARED_PTR_QCLASS General::SharedPtr

  using SHARED_PTR_NAMESPACE::SharedPtr;
  using SHARED_PTR_NAMESPACE::WeakPtr;

  using SHARED_PTR_NAMESPACE::StaticPointerCast;
  using SHARED_PTR_NAMESPACE::DynamicPointerCast;
#else
  template< class T>
  class SharedPtr
    : public SHARED_PTR_NAMESPACE::shared_ptr< T >
  {
  public:
    inline SharedPtr( )
    {
    }

    inline SharedPtr( T* Ptr )
      : SHARED_PTR_NAMESPACE::shared_ptr< T >( Ptr )
    {
    }

    inline SharedPtr( SharedPtr< T > const & Ptr )
      : SHARED_PTR_NAMESPACE::shared_ptr< T >( Ptr )
    {
    }

    inline SharedPtr( SHARED_PTR_NAMESPACE::shared_ptr< T > const & Ptr )
      : SHARED_PTR_NAMESPACE::shared_ptr< T >( Ptr )
    {
    }
  };
  
  template< class T>
  class SharedConstPtr
    : public SharedPtr< T >
  {
  public:
    inline SharedConstPtr( )
    {
    }

    inline SharedConstPtr( T* Ptr )
      : SharedPtr< T >( Ptr )
    {
    }

    inline SharedConstPtr( SharedPtr< T > const & Ptr )
      : SharedPtr< T >( Ptr )
    {
    }

    inline SharedConstPtr( SHARED_PTR_NAMESPACE::shared_ptr< T > const & Ptr )
      : SharedPtr< T >( Ptr )
    {
    }

    inline
    const T& operator*( ) const
    {
      return SharedPtr< T >::operator*( );
    }

    inline
    const T* operator->( ) const
    {
      return SharedPtr< T >::operator->( );
    }

    inline
    const T* get( ) const
    {
      return SharedPtr< T >::get( );
    }
  };
  
  template< class T>
  class WeakPtr
    : public SHARED_PTR_NAMESPACE::weak_ptr< T >
  {
  public:
    inline WeakPtr( )
    {
    }

    inline WeakPtr( SharedPtr< T > Ptr )
      : SHARED_PTR_NAMESPACE::weak_ptr< T >( Ptr )
    {
    }
  };
#endif /* 0 */  
#endif /* !defined(SWIG) */
}
#endif /* ! defined(SWIG) */

#endif /* GENERAL__SHARED_PTR_HH */
